@@ -1,4 +1,4 @@
-# Copyright (c) 2015-2021 by Rocky Bernstein
+# Copyright (c) 2015-2021, 2023 by Rocky Bernstein
 # Copyright (c) 2000-2002 by hartmut Goebel <h.goebel@crazy-compilers.com>
 #
 #  This program is free software; you can redistribute it and/or
@@ -30,15 +30,15 @@ import io
 import sys
 from struct import unpack
 
-from xdis.magics import magic_int2tuple
 from xdis.codetype import to_portable
-from xdis.version_info import PYTHON3, PYTHON_VERSION, PYTHON_VERSION_TRIPLE, IS_PYPY
+from xdis.cross_types import LongTypeForPython3, UnicodeForPython3
+from xdis.magics import magic_int2tuple
+from xdis.version_info import PYTHON3, PYTHON_VERSION_TRIPLE
 
 if PYTHON3:
 
     def long(n):
-        return n
-
+        return LongTypeForPython3(n)
 
 else:
     import unicodedata
@@ -53,8 +53,9 @@ else:
 FLAG_REF = 0x80
 
 
-# The keys in following dictionary are an unmashal codes, like "s", "c", "<", etc.
-# the values of the dictionary are names of routines to call that do the data unmarshaling.
+# The keys in following dictionary are an unmashal codes, like "s",
+# "c", "<", etc.  the values of the dictionary are names of routines
+# to call that do the data unmarshaling.
 #
 # Note: we could eliminate the parameters, if this were all inside a
 # class.  This might be good from an efficiency standpoint, and bad
@@ -111,7 +112,7 @@ def compat_str(s: str) -> str:
 
 
 def compat_u2s(u):
-    if PYTHON_VERSION < 3.0:
+    if PYTHON_VERSION_TRIPLE < (3, 0):
         # See also unaccent.py which can be found using google. I
         # found it and this code via
         # https://www.peterbe.com/plog/unicode-to-ascii where it is a
@@ -201,7 +202,7 @@ class _VersionIndependentUnmarshaller:
         """
         byte1 = ord(self.fp.read(1))
 
-        # FLAG_REF indiates whether we "intern" or
+        # FLAG_REF indicates whether we "intern" or
         # save a reference to the object.
         # byte1 without that reference is the
         # marshal type code, an ASCII character.
@@ -212,7 +213,8 @@ class _VersionIndependentUnmarshaller:
             byte1 = byte1 & (FLAG_REF - 1)
         marshal_type = chr(byte1)
 
-        # print(marshalType) # debug
+        # print(marshal_type)  # debug
+
         if marshal_type in UNMARSHAL_DISPATCH_TABLE:
             func_suffix = UNMARSHAL_DISPATCH_TABLE[marshal_type]
             unmarshal_func = getattr(self, "t_" + func_suffix)
@@ -221,7 +223,7 @@ class _VersionIndependentUnmarshaller:
             try:
                 sys.stderr.write(
                     "Unknown type %i (hex %x) %c\n"
-                    % (ord(marshal_type), hex(ord(marshal_type)), marshal_type)
+                    % (ord(marshal_type), ord(marshal_type), marshal_type)
                 )
             except TypeError:
                 sys.stderr.write(
@@ -261,9 +263,13 @@ class _VersionIndependentUnmarshaller:
         d = long(0)
         for j in range(0, size):
             md = int(unpack("<h", self.fp.read(2))[0])
+            # This operation and turn "d" from a long back
+            # into an int.
             d += md << j * 15
+            d = long(d)
         if n < 0:
             d = long(d * -1)
+
         return self.r_ref(d, save_ref)
 
     # Python 3.4 removed this.
@@ -283,10 +289,14 @@ class _VersionIndependentUnmarshaller:
         return self.r_ref(float(unpack("<d", self.fp.read(8))[0]), save_ref)
 
     def t_complex(self, save_ref, bytes_for_s=False):
-        if self.magic_int <= 62061:
-            get_float = lambda: float(self.fp.read(unpack("B", self.fp.read(1))[0]))
-        else:
-            get_float = lambda: float(self.fp.read(unpack("<i", self.fp.read(4))[0]))
+        def unpack_pre_24() -> float:
+            return float(self.fp.read(unpack("B", self.fp.read(1))[0]))
+
+        def unpack_newer() -> float:
+            return float(self.fp.read(unpack("<i", self.fp.read(4))[0]))
+
+        get_float = unpack_pre_24 if self.magic_int <= 62061 else unpack_newer
+
         real = get_float()
         imag = get_float()
         return self.r_ref(complex(real, imag), save_ref)
@@ -356,18 +366,8 @@ class _VersionIndependentUnmarshaller:
     def t_unicode(self, save_ref, bytes_for_s=False):
         strsize = unpack("<i", self.fp.read(4))[0]
         unicodestring = self.fp.read(strsize)
-        if PYTHON_VERSION_TRIPLE == (3, 2) and IS_PYPY:
-            # FIXME: this isn't quite right. See
-            # pypy3-2.4.0/lib-python/3/email/message.py
-            # '([^\ud800-\udbff]|\A)[\udc00-\udfff]([^\udc00-\udfff]|\Z)')
-            return self.r_ref(unicodestring.decode("utf-8", errors="ignore"), save_ref)
-        else:
-            try:
-                return self.r_ref(unicodestring.decode("utf-8"), save_ref)
-            except UnicodeDecodeError as e:
-                return self.r_ref(
-                    unicodestring.decode("utf-8", errors="ignore"), save_ref
-                )
+        if PYTHON_VERSION_TRIPLE >= (3, 0):
+            return self.r_ref(UnicodeForPython3(unicodestring), save_ref)
 
     # Since Python 3.4
     def t_small_tuple(self, save_ref, bytes_for_s=False):
@@ -511,7 +511,7 @@ class _VersionIndependentUnmarshaller:
         else:
             # < 1.5 there is no lnotab, so no firstlineno.
             # SET_LINENO is used instead.
-            co_firstlineno = -1  # Bogus sentinal value
+            co_firstlineno = -1  # Bogus sentinel value
             co_lnotab = ""
 
         code = to_portable(
