@@ -1,4 +1,4 @@
-# (C) Copyright 2020-2021 by Rocky Bernstein
+# (C) Copyright 2020-2021, 2023 by Rocky Bernstein
 #
 #  This program is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU General Public License
@@ -14,44 +14,35 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-from xdis.version_info import PYTHON_VERSION_TRIPLE
-from xdis.codetype.base import CodeBase
 from copy import deepcopy
 
-# If there is a list of types, then any will work, but the 1st one is the corect one for types.CodeType
-if PYTHON_VERSION_TRIPLE <= (2, 7):
-    Code13FieldTypes = {
-        "co_argcount": int,
-        "co_nlocals": int,
-        "co_flags": int,
-        "co_code": (str, bytes, list, tuple),
-        "co_consts": (tuple, list),
-        "co_names": (tuple, list),
-        "co_varnames": (tuple, list),
-        "co_filename": (str, unicode),
-        "co_name": (str, unicode),
-    }
-else:
-    Code13FieldTypes = {
-        "co_argcount": int,
-        "co_nlocals": int,
-        "co_flags": int,
-        "co_code": (str, bytes, list, tuple),
-        "co_consts": (tuple, list),
-        "co_names": (tuple, list),
-        "co_varnames": (tuple, list),
-        "co_filename": (str, bytes),
-        "co_name": (str, bytes),
-    }
+from xdis.codetype.base import CodeBase
+from xdis.cross_types import UnicodeForPython3
+
+# If there is a list of types, then any will work, but the 1st one is
+# the correct one for types.CodeType.
+Code13FieldTypes = {
+    "co_argcount": int,
+    "co_nlocals": int,
+    "co_flags": int,
+    "co_code": (str, bytes, list, tuple),
+    "co_consts": (tuple, list),
+    "co_names": (tuple, list),
+    "co_varnames": (tuple, list),
+    "co_filename": (str, bytes, UnicodeForPython3),
+    "co_name": (str, bytes, UnicodeForPython3),
+}
 
 
 class Code13(CodeBase):
-    """Class for a Python 1.0 .. 1.4 code object used for Python interpreters other than 1.0 .. 1.4
+    """Class for a Python 1.0 .. 1.4 code object used for Python
+    interpreters other than 1.0 .. 1.4
 
     For convenience in generating code objects, fields like
     `co_consts`, co_names which are (immutable) tuples in the end-result can be stored
     instead as (mutable) lists. Likewise the line number table `co_lnotab`
     can be stored as a simple list of offset, line_number tuples.
+
     """
 
     def __init__(
@@ -76,7 +67,7 @@ class Code13(CodeBase):
         self.co_filename = co_filename
         self.co_name = co_name
         self.fieldtypes = Code13FieldTypes
-        if type(self) == Code13:
+        if type(self) is Code13:
             self.check()
         return
 
@@ -110,7 +101,7 @@ class Code13(CodeBase):
         """
         Return a copy of the code object with new values for the specified fields.
 
-        This is analoguous to the method added to types.CodeType in Python 3.8.
+        This is analogous to the method added to types.CodeType in Python 3.8.
         """
         code = deepcopy(self)
         for field, value in kwargs.items():
