@@ -13,25 +13,29 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-import marshal, py_compile, sys, tempfile, types
-from struct import unpack, pack
-from datetime import datetime
+import marshal
 import os.path as osp
+import py_compile
+import sys
+import tempfile
+import types
+from datetime import datetime
+from struct import pack, unpack
 
 import xdis.marsh
 import xdis.unmarshal
-from xdis.version_info import PYTHON3, PYTHON_VERSION_TRIPLE
+from xdis.dropbox.decrypt25 import fix_dropbox_pyc
 from xdis.magics import (
     IS_PYPY3,
     PYTHON_MAGIC_INT,
     int2magic,
-    magic_int2tuple,
     magic2int,
+    magic_int2tuple,
     magicint2version,
     py_str2tuple,
     versions,
 )
-from xdis.dropbox.decrypt25 import fix_dropbox_pyc
+from xdis.version_info import PYTHON3, PYTHON_VERSION_TRIPLE
 
 
 def is_python_source(path):
@@ -93,7 +97,7 @@ def check_object_path(path):
 
 def is_pypy(magic_int, filename):
     # PyPy 3.8 starts pyston's trend of using Python's magic numbers.
-    if magic_int in (3413,) and filename.endswith("pypy38.pyc"):
+    if magic_int in (3413, 3414) and filename.endswith("pypy38.pyc"):
         return True
     return magic_int in ((62211 + 7, 3180 + 7) + IS_PYPY3)
 
@@ -141,7 +145,7 @@ def load_module(filename, code_objects=None, fast_load=False, get_code=True):
                      `False`.
 
     Return values are as follows:
-        float_tuple    a tuple version number for the given magic_int,
+        version_tuple: a tuple version number for the given magic_int,
                        e.g. (2, 7) or (3, 4)
         timestamp: int; the seconds since EPOCH of the time of the bytecode creation, or None
                         if no timestamp was stored
@@ -305,7 +309,7 @@ def load_module_from_file_object(
                 pass
             else:
                 co = None
-        except:
+        except Exception:
             kind, msg = sys.exc_info()[0:2]
             import traceback
 
