@@ -1,4 +1,4 @@
-# (C) Copyright 2018, 2020 by Rocky Bernstein
+# (C) Copyright 2018, 2020, 2023 by Rocky Bernstein
 #
 #  This program is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU General Public License
@@ -14,10 +14,13 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-import os, marshal, tempfile
-from xdis.magics import MAGIC, PYTHON_MAGIC_INT, int2magic
-from xdis.version_info import PYTHON3, PYTHON_VERSION, IS_PYPY
+import marshal
+import os
+import tempfile
+
 from xdis.load import load_module
+from xdis.magics import MAGIC, PYTHON_MAGIC_INT, int2magic
+from xdis.version_info import IS_PYPY, PYTHON3, PYTHON_VERSION_TRIPLE
 
 
 def wr_long(f, x):
@@ -41,7 +44,7 @@ def dump_compile(codeobject, filename, timestamp, magic):
     codeobject: code object
     filefile: bytecode file to write
     timestamp: timestamp to put in file
-    magic: Pyton bytecode magic
+    magic: Python bytecode magic
     """
     # Atomically write the pyc/pyo file.  Issue #13146.
     # id() is used to generate a pseudo-random filename.
@@ -100,7 +103,7 @@ def compare_bytecode_files(bc_file1, bc_file2):
     bytes2 = f.read()
     f.close
 
-    if PYTHON_VERSION == 3.2 and IS_PYPY:
+    if PYTHON_VERSION_TRIPLE[:2] == (3, 2) and IS_PYPY:
         assert bytes1[4:] == bytes2[4:], "bytecode:\n%s\nvs\n%s" % (
             bytes1[4:],
             bytes2[4:],
@@ -128,12 +131,12 @@ def verify_file(real_source_filename, real_bytecode_filename):
     if not os.path.exists(real_source_filename):
         return
 
-    if PYTHON_VERSION < 3.0:
+    if PYTHON_VERSION_TRIPLE < (3, 0):
         f = open(real_source_filename, "U")
-    elif PYTHON_VERSION == 3.0:
+    elif PYTHON_VERSION_TRIPLE[:2] == (3, 0):
         # Too hard to get working on 3.0
         return
-    elif 3.1 <= PYTHON_VERSION <= 3.4:
+    elif (3, 1) <= PYTHON_VERSION_TRIPLE <= (3, 5):
         f = open(real_source_filename, "rb")
     else:
         f = open(real_source_filename, newline=None, errors="backslashreplace")
